----------------------------------------------------------------------
--
-- Design unit:   Control logic for an application accessible by
--                USB using packetizer and depacketizer
--
-- File name:     applic_ctrl.vhd
--
-- Description:   The block controls the request of number of oscillations
--						of selected Tero cells (one in each blocks)
--
-- Parameters:
--
-- System:        VHDL'93
--
-- Autor:         Cdric Marchand, Hubert Curien Laboratory
--
-- Copyright:     LabHC, 2012
--
-- Revision:      Version 1.00, December 2012
--
----------------------------------------------------------------------

LIBRARY ieee;
USE ieee.std_logic_unsigned.all;
USE ieee.std_logic_1164.all;
USE ieee.std_logic_arith.all;

LIBRARY WORK;
USE WORK.lab_hc_pkg.ALL;

ENTITY applic_ctrl IS
  PORT (
  -- Global clock inputs
    clk_ctrl         : IN STD_LOGIC;  		-- Slow clock signal

  -- Global reset input
    poweron_rst      : IN STD_LOGIC;  		-- Power on reset
 
  -- Signals from/to the bus interface
    data2appl        : IN SLV_127_0;  		-- Data to appl
    wr_data2appl     : IN STD_LOGIC;  		-- Writes data to appl
    --
    data2bus         : OUT SLV_127_0; 		-- Data from appl
    rd_data2bus      : IN STD_LOGIC;  		-- Reads data from appl
    --
    wr_ctrl2appl     : IN STD_LOGIC;  		-- Control word validation

  -- Control word bits
    mode_rst         : IN STD_LOGIC := '0'; 	-- Reset of the appl mode
    appl_rst         : IN STD_LOGIC := '0'; 	-- Reset of the appl main state machine
    mode2appl        : IN SLV_2_0 := "000"; 	-- Mode input from bus
	 cfg_tero		  	: IN SLV_15_0;
	 cfg_acq_time	  	: IN SLV_15_0;

  -- State word bits                                         
    version2bus      : OUT SLV_15_0;         -- Version of the appl
    busy2state       : OUT STD_LOGIC;        -- "Busy to state" signal to bus
    status2bus       : OUT SLV_3_0;          -- Status of the appl
	tero_adr		 		: OUT SLV_15_0;			-- selected tero cell
	acq_time		 		: OUT SLV_15_0;			-- Time of the acquisition window in clock cycle

  -- Signals from application module
    appl_data        : IN STD_LOGIC_VECTOR(31 downto 0);    -- Data from application module
      
  -- Signals to application module
	 enable_tero	 	: OUT STD_LOGIC; 			-- Signal to control the start and stop of Tero cells oscillations
	 data_req		 	: OUT STD_LOGIC; 			-- Request for data
	 select_tero_1	 	: OUT SLV_6_0;				-- Selection of the Tero cell of the first block
	 select_tero_2	 	: OUT SLV_6_0				-- Selection of the Tero cell of the second block
    );
END applic_ctrl;

ARCHITECTURE rtl OF applic_ctrl IS

--
  SIGNAL mode        			: SLV_2_0 := "000";
  SIGNAL reset			 			: std_logic;
  SIGNAL cfg_done					: STD_LOGIC;
  signal CNT_MAX      			: STD_LOGIC_VECTOR(15 DOWNTO 0) := (others => '1');
  signal cfg_tero_s				: SLV_15_0;
  signal cfg_acq_time_s			: SLV_15_0;
  signal select_tero_s			: SLV_15_0;
  signal captur_window      	: STD_LOGIC_VECTOR(15 DOWNTO 0) := (others => '0');

-- appl states - state machine - controls modes and commands
  TYPE SM_APPL_TYPE IS (
	appl_idle,
   appl_config,
	appl_start,
	appl_read,
	appl_err
  );
  signal current_state, next_state: SM_APPL_TYPE;

BEGIN
  version2bus <= VERSION;
  
  reset <= mode_rst or appl_rst or poweron_rst;
-- current state generation
PROCESS(reset, clk_ctrl)
BEGIN
	IF reset = '1' THEN
		current_state <= appl_idle;
	ELSIF rising_edge(clk_ctrl) THEN
		current_state <= next_state;
	END IF;
END PROCESS;

-- Generation of application state
PROCESS(clk_ctrl, reset)
  BEGIN
    IF (reset='1') THEN
      next_state <= appl_idle;
    ELSIF rising_edge(clk_ctrl) THEN
      CASE current_state IS
        WHEN appl_idle 	 =>
          CASE mode IS
				WHEN MODE_CONFIG=>
					next_state  	<= appl_config;
					status2bus 		<= USS_CONFIG;
					tero_adr		<= (others => '0');
					acq_time		<= (others => '0');
					cfg_tero_s 		<= cfg_tero;
					cfg_acq_time_s 	<= cfg_acq_time;
				WHEN MODE_START	=>
					next_state  	<= appl_start;
					status2bus 		<= USS_START;
					tero_adr		<= select_tero_s;
					acq_time		<= CNT_MAX;
				WHEN OTHERS 	=>
					next_state  	<= appl_idle;
					status2bus 		<= USS_IDLE;
					tero_adr		<= (others => '0');
					acq_time		<= (others => '0');
					cfg_tero_s 		<= (others => '0');
					cfg_acq_time_s 	<= (others => '1');
				END CASE;
        WHEN appl_config =>
				CASE mode IS
				WHEN MODE_START	=>
					next_state  	<= appl_start;
					status2bus 		<= USS_START;
					tero_adr		<= select_tero_s;
					acq_time		<= CNT_MAX;
				WHEN OTHERS 	=>
					next_state  	<= appl_config;
					status2bus 		<= USS_CONFIG;
					tero_adr		<= (others => '0');
					acq_time		<= (others => '0');
				END CASE;
        WHEN appl_start  =>
				CASE mode IS
				WHEN MODE_CONFIG=>
					next_state  	<= appl_config;
					status2bus 		<= USS_CONFIG;
					tero_adr		<= (others => '0');
					acq_time		<= (others => '0');
					cfg_tero_s 		<= cfg_tero;
					cfg_acq_time_s 	<= cfg_acq_time;
				WHEN OTHERS 	=>
					IF captur_window = CNT_MAX THEN
						next_state  <= appl_read;
						status2bus 	<= USS_READ;
						tero_adr	<= select_tero_s;
						acq_time	<= CNT_MAX;
					ELSE
						next_state  <= appl_start;
						status2bus 	<= USS_START;
						tero_adr	<= select_tero_s;
						acq_time	<= CNT_MAX;
					END IF;
				END CASE;
		  WHEN appl_read  =>
				IF mode = MODE_IDLE THEN
					next_state  	<= appl_idle;
					status2bus 		<= USS_IDLE;
					tero_adr		<= (others => '0');
					acq_time		<= (others => '0');
					cfg_tero_s 		<= (OTHERS => '0');
					cfg_acq_time_s 	<= (OTHERS => '1');
				ELSE
					next_state  	<= appl_start;
					status2bus 		<= USS_START;
					tero_adr		<= select_tero_s;
					acq_time		<= CNT_MAX;
				END IF;
        WHEN OTHERS 		 =>
          next_state  			<= appl_err;
          status2bus  			<= ERR;
			 tero_adr			<= (others => '0');
			 acq_time			<= (others => '0');
			 cfg_tero_s 		<= (OTHERS => '0');
			 cfg_acq_time_s 	<= (OTHERS => '1');
      END CASE;
    END IF;
END PROCESS;

-----
-- signals generation
-----
-- Busy2bus, captur_window, enable_tero
PROCESS(current_state, clk_ctrl, reset)
BEGIN
	IF reset = '1' THEN
		busy2state <= '0';
		mode <= MODE_IDLE;
		captur_window <= (others => '0');
		enable_tero <= '0';
	ELSIF rising_edge(clk_ctrl) THEN
		IF (wr_ctrl2appl = '1' OR (current_state = appl_config AND cfg_done = '0')) THEN
			busy2state <= '1';
		ELSIF current_state = appl_start THEN
			busy2state <= '1';
			enable_tero <= '1';
			captur_window <= captur_window + 1;
		ELSE
			busy2state <= '0';
			enable_tero <= '0';
			captur_window <= (others => '0');
		END IF;
		mode <= mode2appl;
	END IF;
END PROCESS;

-- Config
PROCESS(reset, current_state, clk_ctrl)
BEGIN
	IF reset = '1' THEN
		select_tero_s	<= (OTHERS => '0');
		CNT_MAX			<= (OTHERS => '1');
		cfg_done			<= '0';
	ELSIF rising_edge(clk_ctrl) THEN
		IF (current_state = appl_config) THEN
			select_tero_s	<= cfg_tero_s;
			CNT_MAX			<= cfg_acq_time_s;
			cfg_done			<= '1';
		END IF;
	END IF;
END PROCESS;

select_tero_1	<=	select_tero_s(6 downto 0);
select_tero_2	<=	select_tero_s(14 downto 8);

-- data_gen, data_raq
PROCESS(reset, current_state, clk_ctrl)
BEGIN
	IF reset = '1' THEN
		data2bus <= (OTHERS => '0');
		data_req <= '0';
	ELSIF rising_edge(clk_ctrl) THEN
		IF current_state = appl_read THEN
			data_req <= '1';
			data2bus(31 downto 0)  <= appl_data;
		ELSE
			data_req <= '0';
		END IF;
	END IF;
END PROCESS;

END rtl;