---------------------------------------------------------------------------
-- Design unit: Interface between EZ-USB FX2 and input/output FIFO or    --
-- depacketizer/packetizer                                               --
---------------------------------------------------------------------------
-- Filename: cypress_if.vhd                                              --
---------------------------------------------------------------------------
-- System: VHDL'93                                                       --
---------------------------------------------------------------------------
-- Authors:                                                              --
-- Ban J., Fischer V.                                                    --
-- (C) 2006, 2007 MICRONIC s.r.o.                                        --
---------------------------------------------------------------------------
-- Description:                                                          --
-- Data interface between Cypress EZ-USB2 interface FX2 and input/output --
-- FIFOs or depacketizer/packetizer. It controls data exchange protocol  --
-- and generates a timeout signal (pcktend).                             --
---------------------------------------------------------------------------
-- Parameters:                                                           --
--    * no parameters in this version                                    --
---------------------------------------------------------------------------
-- Inputs :                                                              --
--    * clk          : system clock                                      --
--    * reset        : global reset of the interface                     --
--    * n_empty_flag : USB2local FIFO in FX2 empty flag (active low)     --
--    * n_full_flag  : local2USB FIFO in FX2 full flag (active low)      --
--    * dp_full      : input FIFO in FPGA (or depacketizer) full flag    --
--                     active when FIFO (or depacketizer) is not ready   --
--    * p_empty      : output FIFO in FPGA (or packetizer) empty flag    --
--                     active when data from FIFO (or packetizer) are    --
--                     not available                                     --
--    * p_data       : 16-bit data from FPGA output FIFO or packetizer   --
--                                                                       --
-- Inputs/outputs:                                                       --
--    * fd           : 16-bit bi-directional data bus                    --
--                                                                       --
-- Outputs :                                                             --
--    * n_sloe       : FX2 data bus output enable                        --
--    * n_slrd       : FX2 USB2local FIFO read enable signal             --
--    * n_slwr       : FX2 local2USB FIFO write enable signal            --
--    * fifo_adr     : 2-bit FIFO select: "00" USB2local, "10" local2USB --
--    * n_pktend     : forced flush of local2USB FIFO in FX2             --
--    * dp_data      : FPGA input FIFO or depacketizer data              --
--    * dp_wr_req    : FPGA input FIFO or depacketizer write request     --
--    * p_rd_req     : FPGA output FIFO or packetizer read request       --
--    * timeout      : timeout signal (only for debugging)               --
--                                                                       --
---------------------------------------------------------------------------
-- Version: 2.00 July 2007                                               --
---------------------------------------------------------------------------
-- Revision history:                                                     --
-- Modifications in Version 2.00 (made by VF in July 2007)               --
--                    - Modified header                                  --
--                    - Added timeout signal for debugging               --
--                    - Timeout counter reset each time the USB          --
--                       interface has taken up activity                 --
--                                                                       --
-- Remarks:   -------------------------------------------------------------
-- A pll should be used to output the interface clock (ifclk) to the     --
-- ez-usb. It is necessary to make ifclk 5 ns earlier than clk.          --
--                                                                       --
-- Cypress_if reverses the byte order of the 16-bit words                --
-- (because ez-usb makes the first byte low and the second one high)     --
-- order of bytes sent by computer:                                      --
--   00 11 22 33 44 55 66 77 88 99 aa bb cc dd ee ff                     --
-- 16-bit words read from ez-usb:                                        --
--   1100  3322  5544  7766  9988  bbaa  ddcc  ffee                      --
-- 16-bit words written to depacketizer:                                 --
--   0011  2233  4455  6677  8899  aabb  ccdd  eeff                      --
-- 128-bit data word created by depacketizer:                            --
--   00112233445566778899aabbccddeeff                                    --
---------------------------------------------------------------------------

LIBRARY ieee;
USE ieee.std_logic_1164.all;

ENTITY cypress_if IS
  PORT (
    -- global
    reset        : IN STD_LOGIC;
    clk          : IN STD_LOGIC;
    -- ez-usb side
    n_empty_flag : IN STD_LOGIC;
    n_full_flag  : IN STD_LOGIC;
    fd           : INOUT STD_LOGIC_VECTOR(15 DOWNTO 0);
    n_sloe       : OUT STD_LOGIC;
    n_slrd       : OUT STD_LOGIC;
    n_slwr       : OUT STD_LOGIC;
    fifo_adr     : OUT STD_LOGIC_VECTOR(1 DOWNTO 0);
    n_pktend     : OUT STD_LOGIC;
    -- to depacketizer or a 16-bit fifo
    dp_full      : IN STD_LOGIC;
    dp_data      : OUT STD_LOGIC_VECTOR(15 DOWNTO 0);
    dp_wr_req    : OUT STD_LOGIC;
    -- from packetizer or a 16-bit show-ahead (!) fifo
    p_empty      : IN STD_LOGIC;
    p_data       : IN STD_LOGIC_VECTOR(15 DOWNTO 0);
    p_rd_req     : OUT STD_LOGIC;
    -- from sequencer
    pckt_end     : IN STD_LOGIC;
    --
    timeout      : OUT STD_LOGIC
  );
END cypress_if;

ARCHITECTURE a OF cypress_if IS

  COMPONENT elapsed IS
    GENERIC (
      log2n   : INTEGER := 5
    );
    PORT (
      start   : IN STD_LOGIC;
      clk     : IN STD_LOGIC;
      timeout : OUT STD_LOGIC
    );
  END COMPONENT;

  TYPE state_protocol IS (sm_idle,           -- nothing to do
                          sm_usb2appl_setup, -- setup for out
                          sm_usb2appl_main,  -- move data from ez-usb to depacketizer
                          sm_appl2usb_setup, -- setup for in
                          sm_appl2usb_main,  -- move data from packetizer to ez-usb
                          sm_pkt_end);       -- n_pktend strobe
  SIGNAL sm_protocol  : state_protocol;
  --
  SIGNAL request_appl2usb      : STD_LOGIC;  -- data is available to be moved in
  SIGNAL request_usb2appl      : STD_LOGIC;  -- data is available to be moved out
  SIGNAL unfinished_appl2usb   : STD_LOGIC;  -- unfinished_appl2usb: there is data in 
                                          -- the ez-usb in buffer that hasn't
                                          -- been sent as a packet yet
  SIGNAL n_unfinished_appl2usb : STD_LOGIC;  -- unfinished_appl2usb: there is data in 
  SIGNAL appl2usb_elapsed      : STD_LOGIC;  -- unfinished_appl2usb has been high for some time
  SIGNAL pckt_end_r           : STD_LOGIC;   -- registered 'packet end' signal from sequencer
  
BEGIN
  n_unfinished_appl2usb <= NOT unfinished_appl2usb;
  
  -- transfer request signals
  request_appl2usb <= '1' WHEN (p_empty = '0') and (n_full_flag = '1') ELSE '0';
  request_usb2appl <= '1' WHEN (dp_full = '0') and (n_empty_flag = '1') ELSE '0';

  -- state machine
  PROCESS(reset, clk) BEGIN
    IF reset = '1' THEN
      sm_protocol <= sm_idle;
      unfinished_appl2usb <= '0';
    ELSIF rising_edge(clk) THEN
      case sm_protocol is
        WHEN sm_idle =>          -- wait for something to do
          IF request_usb2appl = '1' THEN 
            sm_protocol <= sm_usb2appl_setup;
            unfinished_appl2usb <= '0';   -- Version 2.00
          ELSIF request_appl2usb = '1' THEN 
            sm_protocol <= sm_appl2usb_main;    -- fifoadr is already set up
            unfinished_appl2usb <= '0';   -- Version 2.00
          ELSIF appl2usb_elapsed = '1' THEN 
            sm_protocol <= sm_pkt_end;
          END IF;

        WHEN sm_usb2appl_setup =>        -- setup time for n_sloe = 0, fifoadr = "00"
          sm_protocol <= sm_usb2appl_main;

        WHEN sm_usb2appl_main =>         -- read data from ez-usb
          IF request_usb2appl = '0' THEN
            IF request_appl2usb = '1' THEN
              sm_protocol <= sm_appl2usb_setup; -- quick direction switch
            ELSE 
              sm_protocol <= sm_idle;
            END IF;
          END IF;

        WHEN sm_appl2usb_setup =>         -- setup time for fifo_adr = "10"
          sm_protocol <= sm_appl2usb_main;

        WHEN sm_appl2usb_main =>          -- write data to ez-usb
          IF request_appl2usb = '0' THEN
            IF request_usb2appl = '1' THEN
              sm_protocol <= sm_usb2appl_setup;  -- quick direction switch
            ELSE 
              sm_protocol <= sm_idle;
              unfinished_appl2usb <= p_empty and n_full_flag; -- only allow "packet end"
            END IF;                                          -- if it's really necessary
          END IF;

        WHEN sm_pkt_end =>          -- n_pktend strobe (to send incomplete buffer as packet)
          sm_protocol <= sm_idle;
          unfinished_appl2usb <= '0';

      END CASE;
    END IF;
  END PROCESS;

  -- registering 'packet end' signal from sequencer
  PROCESS(clk)
  BEGIN
    IF rising_edge(clk) THEN
      pckt_end_r <= pckt_end;
    END IF;
  END PROCESS;
  
  timeout <= unfinished_appl2usb;

  -- unfinished_appl2usb_elapsed goes high ~10ms (assuming 30MHz clock) after the last
  -- in transfer of data. this triggers the sending of a n_pktend strobe
i_elapsed: elapsed 
              GENERIC MAP (log2N => 22)
              PORT MAP (clk      => clk, 
                        start    => n_unfinished_appl2usb, 
                        timeout  => appl2usb_elapsed);

  -- common signals
  fifo_adr <= "00" WHEN (sm_protocol = sm_usb2appl_setup) or (sm_protocol = sm_usb2appl_main) ELSE "10";
  dp_data  <= fd(7 DOWNTO 0) & fd(15 DOWNTO 8);
  fd       <= p_data(7 DOWNTO 0) & p_data(15 DOWNTO 8) WHEN (sm_protocol = sm_appl2usb_main)
                ELSE (OTHERS => 'Z');   -- tristate bidirectional bus when not writing to it

  -- out packet signals
  n_sloe    <= '0' WHEN (sm_protocol = sm_usb2appl_setup) or (sm_protocol = sm_usb2appl_main) ELSE '1';
  n_slrd    <= '0' WHEN (sm_protocol = sm_usb2appl_main) and (request_usb2appl = '1') ELSE '1';
  dp_wr_req <= '1' WHEN (sm_protocol = sm_usb2appl_main) and (request_usb2appl = '1') ELSE '0';

  -- in packet signals
  n_slwr    <= '0' WHEN (sm_protocol = sm_appl2usb_main) and (request_appl2usb = '1') ELSE '1';
  p_rd_req  <= '1' WHEN (sm_protocol = sm_appl2usb_main) and (request_appl2usb = '1') ELSE '0';
  
  n_pktend  <= '0' WHEN ((sm_protocol = sm_pkt_end) OR (pckt_end_r = '1')) ELSE '1';

END a;