---------------------------------------------------------------------------
-- Design unit:   Clock generator wrapper for Evariste II FPGA modules   --
---------------------------------------------------------------------------
-- File name:     gen_clk_wrp.vhd                                        --
---------------------------------------------------------------------------
-- System: VHDL'93                                                       --
---------------------------------------------------------------------------
-- Authors:                                                              --
-- Fischer V.                                                            --
-- (C) 2013 Laboratoire Hubert Curien                                    --
---------------------------------------------------------------------------
-- Description:   The block gen_clk_wrp ...                              --
---------------------------------------------------------------------------
-- Parameters:                                                           --
---------------------------------------------------------------------------
-- Inputs :                                                              --
--    * clk_in       : system clock                                      --
--                                                                       --
-- Outputs :                                                             --
--    * clk_ctrl      : Control clock                                    --
--    * clk_applic    : Application clock                                --
--    * clk_usbif_out : USB interface clock (to CYPRESS)                 --
--    * locked        : DCMs are locked                                  --
--                                                                       --
---------------------------------------------------------------------------
-- Version: 0.0.1.01, May 2013                                           --
---------------------------------------------------------------------------
-- Revision history:                                                     --
-- Modifications in Version 1.01 made by VF in ...:                      --
--                                                                       --
-- Remarks:   -------------------------------------------------------------
-- It is necessary to make clk_usbif_out 6 ns earlier than clk_ctrl.     --
---------------------------------------------------------------------------

LIBRARY ieee;
USE ieee.std_logic_1164.ALL;
USE ieee.std_logic_unsigned.ALL;
USE ieee.std_logic_arith.ALL;

LIBRARY work;
USE work.lab_hc_pkg.ALL;

ENTITY gen_clk_wrp IS
PORT
  (
-- Input clock
  clk_in_p        : IN STD_LOGIC;  -- Input clock signal
  clk_in_n        : IN STD_LOGIC;

-- Output clocks
  clk_ctrl      : OUT STD_LOGIC; -- and clk_usbif_int
  clk_applic    : OUT STD_LOGIC;
  clk_usbif_out : OUT STD_LOGIC;
  locked        : OUT STD_LOGIC 
 );
END gen_clk_wrp;

ARCHITECTURE rtl OF gen_clk_wrp IS

-------------------------------------------------------------------------------
-- Declarations of components
-------------------------------------------------------------------------------
-- Clock generator
--------------------------------------------------------------------------
-- Two DCMs generate 3 clocks:
-- c0 - clk_ctrl 
-- c1 - clk_applic 
-- c2 - clk_usbif 
--------------------------------------------------------------------------
component ip_gen_clk_usbif_out is
port
 (-- Clock in ports
  CLK_IN1           : in     std_logic;
  -- Clock out ports
  CLK_OUT1          : out    std_logic;
  -- Status and control signals
  RESET             : in     std_logic;
  LOCKED            : out    std_logic
 );
end component;

COMPONENT gen_clk_ctrl IS
   PORT ( CLKIN_IN        : IN    STD_LOGIC; 
          RST_IN          : IN    STD_LOGIC; 
          CLKFX_OUT       : OUT   STD_LOGIC; 
          CLKIN_IBUFG_OUT : OUT   STD_LOGIC; 
          CLK0_OUT        : OUT   STD_LOGIC; 
          LOCKED_OUT      : OUT   STD_LOGIC);
END COMPONENT;

component ip_gen_clk_s6 is
port
 (-- Clock in ports
  CLK_IN1_P           : in     std_logic;
  CLK_IN1_N           : in     std_logic;
  -- Clock out ports
  CLK_OUT1          : out    std_logic;
  CLK_OUT2          : out    std_logic;
  -- Status and control signals
  RESET             : in     std_logic;
  LOCKED            : out    std_logic
 );
end component;

SIGNAL locked_s     : STD_LOGIC_VECTOR(1 DOWNTO 0);
SIGNAL clk_s        : STD_LOGIC;
 
BEGIN
-- Instantiation of components -----------------------------------------------
-- DCM instantiation  
  -- clk_usbif has to be in the range 5 - 48 MHz
  -- Use these DCMs to choose a frequency. Make clk_usbif ~6ns earlier than clk_ctrl
i_gen_clk_ctrl : ip_gen_clk_s6
  PORT MAP(
          CLK_IN1_P   => clk_in_p,
			 CLK_IN1_N   => clk_in_n,
          RESET     => '0',
          CLK_OUT2  => clk_s, --
          LOCKED => locked_s(0)
          );
 
 i_gen_clk_usbif_out : ip_gen_clk_usbif_out
   PORT MAP(
           CLK_IN1   => clk_s,
           RESET     => '0',
           CLK_OUT1   => clk_usbif_out,
           LOCKED => locked_s(1)
           );

  clk_ctrl   <= clk_s;
  clk_applic <= clk_s;
 
  locked     <= locked_s(1) AND locked_s(0);

END rtl;