---------------------------------------------------------------------------
-- Design unit:   True random number generator                           --
--                with Cypress EZ-USB2 FX2 16-bit interface              -- 
---------------------------------------------------------------------------
-- File name:     transfer2usb.vhd                                           --
---------------------------------------------------------------------------
-- System: VHDL'93                                                       --
---------------------------------------------------------------------------
-- Authors:                                                              --
-- Fischer V.                                                            --
-- (C) 2012 Laboratoire Hubert Curien                                    --
---------------------------------------------------------------------------
-- Description:   The block transfer2usb ...                                 --
---------------------------------------------------------------------------
-- Parameters:                                                           --
---------------------------------------------------------------------------
-- Inputs :                                                              --
--    * clk_in       : system clock                                      --
--    * syst_rst     : system global reset                               --
--    * pwr_detect   : VCC detection                                     --
--                                                                       --
-- Inputs/outputs:                                                       --
--    * fd           : 16-bit bi-directional data bus                    --
--                                                                       --
-- Outputs :                                                             --
--    * res_cyp      : reset Cypress EZ-USB (during FPGA configuration)  --
--    * clk_usbif_out: clock output to the Cypress EZ-USB i/f            --
--    * n_sloe       : FX2 data bus output enable                        --
--    * n_slrd       : FX2 USB2local FIFO read enable signal             --
--    * n_slwr       : FX2 local2USB FIFO write enable signal            --
--    * fifo_adr     : 2-bit FIFO select: "00" USB2local, "10" local2USB --
--    * n_pktend     : forced flush of local2USB FIFO in FX2             --
--                                                                       --
---------------------------------------------------------------------------
-- Version: 0.0.1.01, March 2013                                         --
---------------------------------------------------------------------------
-- Revision history:                                                     --
-- Modifications in Version 1.01 made by VF in ...:                      --
--                                                                       --
-- Remarks:   -------------------------------------------------------------
-- A pll should be used to output the interface clock (ifclk) to the     --
-- ez-usb. It is necessary to make ifclk 5 ns earlier than clk.          --
---------------------------------------------------------------------------

LIBRARY ieee;
USE ieee.std_logic_1164.ALL;
USE ieee.std_logic_unsigned.ALL;
USE ieee.std_logic_arith.ALL;

LIBRARY work;
USE work.lab_hc_pkg.ALL;

ENTITY transfer2usb IS
PORT
  (
-- Local clock signals
  clk_in_p        : IN STD_LOGIC;  -- Input clock signal
  clk_in_n        : IN STD_LOGIC;
  
  --port 0 et 1 des gpio
  syst_rst      : IN STD_LOGIC;  -- System reset signal
  pwr_detect    : IN STD_LOGIC :='1'; -- VCC detection

-- Cypress EZ-USB controller interface
  res_cyp       : OUT STD_LOGIC;
  clk_usbif_out : OUT STD_LOGIC;
  n_empty_flag  : IN STD_LOGIC;
  n_full_flag   : IN STD_LOGIC;
  fd            : INOUT STD_LOGIC_VECTOR(15 DOWNTO 0);
  n_sloe        : OUT STD_LOGIC;
  n_slrd        : OUT STD_LOGIC;
  n_slwr        : OUT STD_LOGIC;
  fifo_adr      : OUT STD_LOGIC_VECTOR(1 DOWNTO 0);
  n_pktend      : OUT std_logic
 );
END transfer2usb;

ARCHITECTURE rtl OF transfer2usb IS

-------------------------------------------------------------------------------
-- Declarations of components
-------------------------------------------------------------------------------
-- Clock generator
--------------------------------------------------------------------------
-- Generator generates 3 clocks:
-- c0 - clk_ctrl 
-- c1 - clk_applic 
-- c2 - clk_usbif_out
--------------------------------------------------------------------------
  COMPONENT gen_clk_wrp IS
  PORT
  (
  -- Input clock
  clk_in_p        : IN STD_LOGIC;  -- Input clock signal
  clk_in_n        : IN STD_LOGIC;

  -- Output clocks
  clk_ctrl      : OUT STD_LOGIC; -- and clk_usbif_int
  clk_applic    : OUT STD_LOGIC;
  clk_usbif_out : OUT STD_LOGIC;
  locked        : OUT STD_LOGIC 
  );
  END COMPONENT;

--------------------------------------------------------------------------
-- Cypres EZ USB controller interface
--------------------------------------------------------------------------

  COMPONENT cypress_if IS
    PORT (
      reset        : IN STD_LOGIC;
      clk          : IN STD_LOGIC;
      n_empty_flag : IN STD_LOGIC;
      n_full_flag  : IN STD_LOGIC;
      fd           : INOUT STD_LOGIC_VECTOR(15 DOWNTO 0);
      n_sloe       : OUT STD_LOGIC;
      n_slrd       : OUT STD_LOGIC;
      n_slwr       : OUT STD_LOGIC;
      fifo_adr     : OUT STD_LOGIC_VECTOR(1 DOWNTO 0);
      n_pktend     : OUT STD_LOGIC;
      dp_full      : IN STD_LOGIC;
      dp_data      : OUT STD_LOGIC_VECTOR(15 DOWNTO 0);
      dp_wr_req    : OUT STD_LOGIC;
      p_empty      : IN STD_LOGIC;
      p_data       : IN STD_LOGIC_VECTOR(15 DOWNTO 0);
      p_rd_req     : OUT STD_LOGIC;
      pckt_end     : IN STD_LOGIC;
      timeout      : OUT STD_LOGIC      
    );
  END COMPONENT;
  
--------------------------------------------------------------------------
-- Sequencer module containing depacketizer, packetizer, "end of packet"
-- and time-out logic.
--------------------------------------------------------------------------
  COMPONENT sequencer IS
    PORT (
    -- global
    clk           : IN STD_LOGIC;
    reset         : IN STD_LOGIC;
    mode_rst      : IN STD_LOGIC;
    
    -- cypress_if modul interface
    dp_data       : IN STD_LOGIC_VECTOR(15 DOWNTO 0);
    dp_wr_req     : IN STD_LOGIC;
    dp_full       : OUT STD_LOGIC;
    --
    p_data        : OUT STD_LOGIC_VECTOR(15 DOWNTO 0);
    p_rd_req      : IN STD_LOGIC;
    p_empty       : OUT STD_LOGIC;
    --
    pckt_end      : OUT STD_LOGIC;
    
  	-- application module interface
    data2appl     : OUT STD_LOGIC_VECTOR(127 DOWNTO 0); -- output to application module
    wr_data2appl  : OUT STD_LOGIC;                      -- data write strobe
    ctrl2appl     : OUT STD_LOGIC_VECTOR(63 DOWNTO 0);  -- output to command lines
    wr_ctrl2appl  : OUT STD_LOGIC;                      -- command write strobe
    --
    data2bus      : IN STD_LOGIC_VECTOR(127 DOWNTO 0);  -- input from application module
    rd_data2bus   : OUT STD_LOGIC;                      -- data read strobe
    state2bus     : IN STD_LOGIC_VECTOR(95 DOWNTO 0);   -- input from status lines

    trans_pending : OUT STD_LOGIC;                      -- a transaction is pending
    
    busy2state    : IN STD_LOGIC                        -- module is busy
    );
  END COMPONENT;
 
---------------------------------------------------------------------
-- Function applic_ctrl represents the control unit of the application. 
-- The main control clock (clk_ctrl) is obtained by a division of 
-- clk_applic by two in the PLL. The "appl_rst" resets main application 
-- state machine (sm_appl). 
-- Signals: ctrl_rst_mode, comm2appl and mode2appl represent different 
-- bits of the application control register situated in the bus interface
-- block. Signals: wr_data2appl, rd_data2bus and wr_ctrl2appl are short 
-- (one period) signals used to write incomming command to the application, 
-- to start data processing (or other action), to get next random number, 
-- etc. 
-- Signal: ctrlif_reset is used to reset control register in bus 
-- interface. Signals: busy2bus, mode2bus and status are output in 
-- the state word of the application.
-- Signals: data2appl and data2bus are input and output data signals
-- of the application (128-bit application data bus).
---------------------------------------------------------------------
  COMPONENT applic_ctrl IS
    PORT (
  -- Global clock inputs
--    clk_applic        : IN STD_LOGIC;  -- Fast clock signal
    clk_ctrl          : IN STD_LOGIC;  -- Slow clock signal

  -- Global reset input
    poweron_rst       : IN STD_LOGIC;  -- Power on reset
 
  -- Signals from/to the bus interface
    data2appl         : IN SLV_127_0;  -- Data to appl
    wr_data2appl      : IN STD_LOGIC;  -- Writes data to appl
    --
    data2bus          : OUT SLV_127_0; -- Data from appl
    rd_data2bus       : IN STD_LOGIC;  -- Reads data from appl
    --
    wr_ctrl2appl      : IN STD_LOGIC;  -- Control word validation

  -- Control word bits
    mode_rst          : IN STD_LOGIC := '0'; -- Reset of the appl mode
    appl_rst          : IN STD_LOGIC := '0'; -- Reset of the appl main state machine
    mode2appl         : IN SLV_2_0 := "000"; -- Mode input from bus
	 cfg_tero			 : IN SLV_15_0;
	 cfg_acq_time		 : IN SLV_15_0;

  -- State word bits                                         
    version2bus      : OUT SLV_15_0;         -- Version of the appl
    busy2state       : OUT STD_LOGIC;        -- "Busy to state" signal to bus
    status2bus       : OUT SLV_3_0;          -- Status of the appl
	 tero_adr			: OUT SLV_15_0;				-- selected tero cell
	 acq_time			: OUT SLV_15_0;			-- Time of the acquisition window in clock cycle

  -- Signals from application module
    appl_data        : IN STD_LOGIC_VECTOR(31 downto 0);    -- Data from application module
      
  -- Signals to application module
	 enable_tero		: OUT STD_LOGIC;
	 data_req			: OUT STD_LOGIC;
	 select_tero_1		: OUT SLV_6_0;
	 select_tero_2		: OUT SLV_6_0
    );
  END COMPONENT;

---------------------------------------------------------------------
-- The block "applic_wrp" includes simple application (a binary counter) 
-- and control signals. 
-- Output signal "appl_ready" is asserted, when random data is available 
-- in the output register.    
-- "reset" resets output and control registers.
---------------------------------------------------------------------
  COMPONENT applic_wrp IS
  PORT(
    clk_ctrl      	: IN STD_LOGIC;           -- Logic clock
    reset         	: IN STD_LOGIC;           -- Resets control registers
-- Signals from controller module
	 enable_tero		: IN STD_LOGIC;
	 data_req			: IN STD_LOGIC;
	 select_tero_1		: IN SLV_6_0;
	 select_tero_2		: IN SLV_6_0;

-- Signals to controller module
    appl_data        : OUT STD_LOGIC_VECTOR(31 downto 0)
    );
  END COMPONENT;
  
-------------------------------------------------------------------------------
-- Declarations of internal signals
-------------------------------------------------------------------------------
-- Clock and global signals
  SIGNAL clk_ctrl            : STD_LOGIC;
  SIGNAL clk_applic          : STD_LOGIC;
  SIGNAL clk_usbif_int       : STD_LOGIC;
--  SIGNAL clk_usbif_out_s       : STD_LOGIC;
  

  SIGNAL system_rst          : STD_LOGIC;

  SIGNAL locked              : STD_LOGIC;

-- Macrofunction "application" IOs
  SIGNAL enable_tero         : STD_LOGIC;
  SIGNAL data_req	           : STD_LOGIC;
  SIGNAL appl_data           : STD_LOGIC_VECTOR (31 DOWNTO 0);
  SIGNAL select_tero_1		  : SLV_6_0;
  SIGNAL select_tero_2		  : SLV_6_0;
  
-- Depacketizer interface signals  
  SIGNAL dp_full       : STD_LOGIC;
  --
  SIGNAL dp_data       : SLV_15_0;
  --
  SIGNAL dp_wr_req     : STD_LOGIC;
  --

-- Packetizer interface signals    
  SIGNAL p_empty      : STD_LOGIC;
  --
  SIGNAL p_data       : STD_LOGIC_VECTOR(15 DOWNTO 0);
  --
  SIGNAL p_rd_req     : STD_LOGIC;
  
  SIGNAL pckt_end     : STD_LOGIC;
  
  SIGNAL trans_pending: STD_LOGIC;
  
-- appl interface signals  
  SIGNAL data2appl           : STD_LOGIC_VECTOR (127 DOWNTO 0);
  SIGNAL wr_data2appl        : STD_LOGIC;
  --
  SIGNAL ctrl2appl           : STD_LOGIC_VECTOR(63 DOWNTO 0);
  SIGNAL wr_ctrl2appl        : STD_LOGIC;
  --
  SIGNAL data2bus            : STD_LOGIC_VECTOR(127 DOWNTO 0);
  SIGNAL rd_data2bus         : STD_LOGIC;
  --
  SIGNAL state2bus           : STD_LOGIC_VECTOR(95 DOWNTO 0) := X"000000000000000000000000";
 
-- Control word bits
  SIGNAL mode_rst            : STD_LOGIC :='0';
  SIGNAL appl_rst            : STD_LOGIC :='0';
  --
  SIGNAL mode2appl           : SLV_2_0 :="000";
  SIGNAL cfg_tero				  : SLV_15_0;
  SIGNAL cfg_acq_time		  : SLV_15_0;

-- State word bits
  SIGNAL busy2state          : STD_LOGIC;
  SIGNAL version2bus         : SLV_15_0;
  SIGNAL status2bus          : SLV_3_0 :="0000";
  
-- Internal signals  
  SIGNAL poweron_rst         : STD_LOGIC :='0';

BEGIN
-- Reset of the Cypress device (during FPGA configuration)
--  res_cyp      <= NOT locked; -- Cypress must wait until FPGA is ready
  res_cyp <= '0';
  
-- Internal reset signals (all active in level one)
  system_rst   <= NOT syst_rst; -- Global reset including Cypress EZ-USB
--  system_rst   <= syst_rst; -- Global reset including Cypress EZ-USB
  poweron_rst  <= NOT pwr_detect; -- VCC detect
  --
  appl_rst    <= ctrl2appl(57) 
                  OR poweron_rst
                    OR system_rst; 
  --
  mode_rst     <= ctrl2appl(58) 
                    OR ctrl2appl(57) 
                      OR poweron_rst
                        OR system_rst;
  --
  state2bus(95 DOWNTO 80) <= version2bus;
  state2bus(79 DOWNTO 36) <= (OTHERS => '0');
--  state2bus(23 DOWNTO 22)  <= (OTHERS => '0');
--  state2bus(7 DOWNTO 4)   <= (OTHERS => '0');
  state2bus(3 DOWNTO 0)   <= status2bus;
  --
  
  mode2appl  	<= ctrl2appl(2 DOWNTO 0);
  cfg_tero	 	<= ctrl2appl(19 DOWNTO 4);
  cfg_acq_time	<= ctrl2appl(35 DOWNTO 20);

-- Instantiation of components -----------------------------------------------
-- Clock generator instantiation  
  -- ifclk has to be in the range 5 - 48 MHz
  -- Use this pll to choose a frequency. Make ifclk ~5ns earlier than clk
i_gen_clk_wrp: gen_clk_wrp PORT MAP (
    clk_in_p        => clk_in_p, 
	 clk_in_n        => clk_in_n,
    clk_ctrl      => clk_ctrl,      -- and clk_usbif_int
    clk_applic    => clk_applic,    -- and clk_rng
    clk_usbif_out => clk_usbif_out, 
    locked        => locked
  );

  clk_usbif_int <= clk_ctrl;

-- Cypress EZ-USB instantiation
i_cypress_if:
  cypress_if PORT MAP (
    clk           => clk_usbif_int,
    reset         => system_rst,
	 
    -- Cypress EZ-USB interface signals    
    n_empty_flag  => n_empty_flag,
    n_full_flag   => n_full_flag,
    fd            => fd,
    n_sloe        => n_sloe,
    n_slrd        => n_slrd,
    n_slwr        => n_slwr,
    fifo_adr      => fifo_adr,
    n_pktend      => n_pktend,
	 
    -- Sequencer interface signals
    dp_full       => dp_full,
    dp_data       => dp_data,
    dp_wr_req     => dp_wr_req,
    --
    p_empty       => p_empty,
    p_data        => p_data,
    p_rd_req      => p_rd_req,
    --
    pckt_end      => pckt_end
  );

-- Sequencer instantiation
i_sequencer: sequencer 
  PORT MAP (
    -- Global signals
    clk           => clk_usbif_int,
    reset          => system_rst,
    mode_rst       => mode_rst,
	 
    -- Cypress EZ-USB interface and serial interface signals
    dp_data        => dp_data,
    dp_wr_req      => dp_wr_req,
    dp_full        => dp_full,
    --
    p_data         => p_data,
    p_rd_req       => p_rd_req,
    p_empty        => p_empty,
    --
    pckt_end       => pckt_end,

    -- Application interface signals
    data2appl      => data2appl,    -- Data output to application module
    wr_data2appl   => wr_data2appl, -- Data write strobe
    --
    ctrl2appl      => ctrl2appl,    -- Control word output to application module
    wr_ctrl2appl   => wr_ctrl2appl, -- Control word write strobe
    --
    data2bus       => data2bus,     -- Data from application module
    rd_data2bus    => rd_data2bus,  -- Data read strobe
    state2bus      => state2bus,    -- State word from application module
    --
    trans_pending  => trans_pending,-- A transaction is pending (not used here)
    --
    busy2state     => busy2state    -- Application module is busy
  );
  
-- Application module instantiation
i_applic_ctrl: applic_ctrl 
  PORT MAP (
  -- Global clock signals
    clk_ctrl            => clk_ctrl,
--    clk_applic          => clk_applic,

  -- Global reset input
    poweron_rst         => poweron_rst,
                    
  -- Signals from/to the bus interface
    data2appl           => data2appl,
    wr_data2appl        => wr_data2appl,
    --
    data2bus            => data2bus,
    rd_data2bus         => rd_data2bus,
    --
    wr_ctrl2appl        => wr_ctrl2appl,
	 
    -- Control word bits
    mode_rst            => mode_rst,
    appl_rst            => appl_rst,
    mode2appl           => mode2appl,
	 cfg_tero				=> cfg_tero,
	 cfg_acq_time			=> cfg_acq_time,
	 
    -- State word bits
    version2bus         => version2bus,
    busy2state          => busy2state, 
    status2bus          => status2bus,
	 tero_adr				=> state2bus(19 DOWNTO 4),
	 acq_time				=> state2bus(35 DOWNTO 20),
                    
  -- Signals from application module
    appl_data           => appl_data,

  -- Signals to application module
	 select_tero_1			=> select_tero_1,
	 select_tero_2			=> select_tero_2,
	 data_req   			=> data_req,
    enable_tero         => enable_tero
    );
 
-- Application instantiation
i_aplic_wrp: applic_wrp
  PORT MAP (
    clk_ctrl      		=> clk_ctrl,
    reset         		=> appl_rst,
	 
    enable_tero   		=> enable_tero,
	 data_req   			=> data_req,
	 select_tero_1			=> select_tero_1,
	 select_tero_2			=> select_tero_2,
    appl_data     		=> appl_data
  );

END rtl;